@@ -20,7 +20,6 @@ class ConceptDrift(AbstractAnomaly):
         self.acc_arr = []
         self.avg_acc = 0
         self.log_handler = fw.log_handler
-        self.log_handler.add_writer(self.dashboard_name)
         if check["algorithm"] == DataDriftAlgo.DDM:
             warn_thres = check.get("warn_thres", 2)
             alarm_thres = check.get("alarm_thres", 3)
@@ -45,13 +44,13 @@ class ConceptDrift(AbstractAnomaly):
             )
             self.log_handler.add_scalars(
                 "avg_accuracy",
-                {"avg_accuracy": self.avg_acc},
+                {"y_avg_accuracy": self.avg_acc},
                 len(self.acc_arr),
                 self.dashboard_name,
             )
             if isinstance(alert, str):
                 self.log_handler.add_alert(
-                    "Concept Drift Alert",
+                    "Model Performance Degradation Alert 🚨",
                     alert,
                     self.dashboard_name
                 )
