@@ -12,4 +12,4 @@ class RecHitRateMeasurable(Measurable):
         return np.array(acc)
 
     def col_name(self):
-        return "Measurable: Hit-Rate"
\ No newline at end of file
+        return "Hit-Rate"
\ No newline at end of file
